package com.capitalware.ufm.action.objects;

import java.util.ArrayList;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class holds the values for the If action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class LoopObject extends UFMObject
{
   public int                 start = 1;
   public int                 end = 1;
   public int                 by = 1;
   public ArrayList<UFMObject> actionAL = null;
   
   /**
    * The constructor
    */
   public LoopObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      
   }

   /**
    * Verify the values in the object.
    */
   public void checkObject()
   {
      if (by == 0)
         throw new IllegalArgumentException("Error: The 'by' attribute for the 'Loop' action cannot be zero.");
      else if (by > 0)
      {
         if (end < start)
            throw new IllegalArgumentException("Error: The 'start' attribute cannot be less than 'end' attribute.");
      }
      else
      {
         if (end > start)
            throw new IllegalArgumentException("Error: The 'end' attribute cannot be less than 'start' attribute.");
      }
   }

   /**
    * Output the info
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("     start                = " + start);
      UFM.logger.info("     end                  = " + end);
      UFM.logger.info("     by                   = " + by);
      
      if (actionAL != null)
      {
         UFM.logger.info("     If Actions to be processed:");
         for (int i=0; i < actionAL.size(); i++)
         {
            UFMObject fmo = (UFMObject)actionAL.get(i);
            UFM.logger.info("       Action ["+(i+1)+"]         = " + fmo.actionName);
         }
      }
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();

      al.add("<"+CWConstants.LOOP_ELEMENT+" "+CWConstants.START_ATTRIBUTE+"=\""+start+"\" "+CWConstants.END_ATTRIBUTE+"=\""+end+"\" "+CWConstants.BY_ATTRIBUTE+"=\""+by+"\">"+LINE_SEPARATOR);

      /*
       * Add info from actions
       */
      if (actionAL != null)
      {
         for (int i=0; i < actionAL.size(); i++)
         {
            UFMObject fmo = (UFMObject)actionAL.get(i);
            ArrayList<String> actionAL = fmo.toStringArray();
            for (int j=0; j < actionAL.size(); j++)
            {
               al.add("  "+actionAL.get(j));
            }
         }
      }
      
      al.add("</"+CWConstants.LOOP_ELEMENT+">"+LINE_SEPARATOR);
      
      return al;
   }

   /**
    * Retrieve the values in this action for the status update.
    * @return s
    */
   public String toStringForStatusUpdate(String event)
   {
      String s;

      s = CWConstants.LOOP_ELEMENT+", "+labelName+", "+CWConstants.LOOP_ELEMENT+", "+CWConstants.START_ATTRIBUTE+"="+start+", "+CWConstants.END_ATTRIBUTE+"="+end+", "+CWConstants.BY_ATTRIBUTE+"="+by+LINE_SEPARATOR;

      return s;
   }
}
