package com.capitalware.ufm.action.objects;

import java.io.File;
import java.util.ArrayList;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class holds the values for the Merge action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MergeObject extends UFMFileObject
{
   public String[]   srcFilenames = null;
   public String     toFile = null;
   public String     toDir = null;
   public boolean    createDir = false;

   /**
    * The constructor
    */
   public MergeObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      if (toDir != null)
         toDir = Tokens.replace(toDir, paClass.actionProperties);
      
      if (toFile != null)
         toFile = Tokens.replace(toFile, paClass.actionProperties);
      
      for (int i=0; i < srcFilenames.length; i++)
         srcFilenames[i] = Tokens.replace(srcFilenames[i], paClass.actionProperties);
   }

   /**
    * Verify the values in the object.
    */
   public void checkObject()
   {
      checkAndCreateDirectory(createDir, toDir);
      
      if (srcFilenames == null)
         throw new IllegalArgumentException("Error: You must specify at least one source FileName for the Merge action.");

      if (toFile == null)
         throw new IllegalArgumentException("Error: You must specify a ToFileName for the Merge action.");
   }

   /**
    * Output the info
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("     Source File Names    = " + srcFilenames[0]);
      for (int i=1; i < srcFilenames.length; i++)
         UFM.logger.info("                            " + srcFilenames[i]);
      
      UFM.logger.info("     Target Directory     = " + toDir);
      UFM.logger.info("     Target File          = " + toFile);
      UFM.logger.info("     Create Directory     = " + createDir);
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();

      if (toDir == null)
         al.add("<"+CWConstants.MERGE_ELEMENT+" "+CWConstants.TO_FILE_ATTRIBUTE+"=\""+toFile+"\">"+LINE_SEPARATOR);
      else
         al.add("<"+CWConstants.MERGE_ELEMENT+" "+CWConstants.TO_DIR_ATTRIBUTE+"=\""+toDir+"\" "+CWConstants.TO_FILE_ATTRIBUTE+"=\""+toFile+"\">"+LINE_SEPARATOR);
      
      for (int i=0; i < srcFilenames.length; i++)
      {
         if (srcFilenames[i].indexOf(File.separator) != -1 )
         {
            int x = srcFilenames[i].lastIndexOf(File.separator);

            String dir   = srcFilenames[i].substring(0, x);
            String fname = srcFilenames[i].substring(x+1);

            al.add("  <"+CWConstants.FILE_ELEMENT+" "+CWConstants.DIR_ATTRIBUTE+"=\""+dir+"\">"+fname+"</"+CWConstants.FILE_ELEMENT+">"+LINE_SEPARATOR);
         }
         else
            al.add("  <"+CWConstants.FILE_ELEMENT+">"+srcFilenames[i]+"</"+CWConstants.FILE_ELEMENT+">"+LINE_SEPARATOR);
      }
      
      al.add("</"+CWConstants.MERGE_ELEMENT+">"+LINE_SEPARATOR);
      
      return al;
   }

   /**
    * Retrieve the values in this action for the status update.
    * @return s
    */
   public String toStringForStatusUpdate(String event)
   {
      String s;

      s = CWConstants.MERGE_ELEMENT+", "+labelName+", "+CWConstants.MERGE_ELEMENT+", "+CWConstants.CREATE_DIR_ATTRIBUTE+"="+createDir+", srcFilenames="+StringUtilities.toStringFromArrayOfString(srcFilenames)+", "+CWConstants.TO_DIR_ATTRIBUTE+"="+toDir+", "+CWConstants.TO_FILE_ATTRIBUTE+"="+toFile+LINE_SEPARATOR;

      return s;
   }
}
