package com.capitalware.ufm.action.objects;

import java.util.ArrayList;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class holds the values for the Schedule action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ScheduleObject extends UFMObject
{
   public String  xmlFile = null;

   /**
    * The constructor
    */
   public ScheduleObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      if (xmlFile != null)
         xmlFile = Tokens.replace(xmlFile, paClass.actionProperties);
   }

   /**
    * Verify the values in the object.
    */
   public void checkObject()
   {
      if (xmlFile == null)
         throw new IllegalArgumentException("Error: You must specify a xmlFile for the SendEmail action.");
   }

   /**
    * Output the info
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("     XML File             = " + xmlFile);
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();
      
      al.add("<"+CWConstants.SCHEDULE_ELEMENT+" "+CWConstants.XML_FILE_ATTRIBUTE+"=\""+xmlFile+"\" />"+LINE_SEPARATOR);
      
      return al;
   }

   /**
    * Retrieve the values in this action for the status update.
    * @return s
    */
   public String toStringForStatusUpdate(String event)
   {
      String s;

      s = CWConstants.SCHEDULE_ELEMENT+", "+labelName+", "+event+", "+CWConstants.XML_FILE_ATTRIBUTE+"="+xmlFile+LINE_SEPARATOR;

      return s;
   }
}
