package com.capitalware.ufm.action.objects;

import java.io.File;
import java.util.ArrayList;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class holds the values for the Split action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class SplitObject extends UFMFileObject
{
   public String     srcFilename = null;
   public String     toDir = null;
   public char       type = 'b';
   public long       size = 0;
   public String     prefix = null;
   public String     extension = null;
   public boolean    createDir = false;

   /**
    * The constructor
    */
   public SplitObject()
   {
      super();
   }

   /**
    * Process tokens in Action Object's variables
    */
   public void performTokenization()
   {
      if (srcFilename != null)
         srcFilename   = Tokens.replace(srcFilename, paClass.actionProperties);
      if (toDir != null)
         toDir = Tokens.replace(toDir, paClass.actionProperties);
      if (prefix != null)
         prefix = Tokens.replace(prefix, paClass.actionProperties);
      if (extension != null)
         extension = Tokens.replace(extension, paClass.actionProperties);
   }

   /**
    * Verify the values in the object.
    */
   public void checkObject()
   {
      String filename = IO.getFileName(srcFilename);

      if (prefix == null)
      {
         if (filename.indexOf('.') != -1)
            prefix = filename.substring(0, filename.lastIndexOf('.'));
         else
            prefix = filename;
      }
      
      checkAndCreateDirectory(createDir, toDir);
      
      if (srcFilename == null)
         throw new IllegalArgumentException("Error: You must specify a filename for the Split action.");

      if (size < 1)
         throw new IllegalArgumentException("Error: You must specify a size attribute that is greater then zero for the Split action.");

      if ( (type != CWConstants.SPLIT_MODE_BINARY) && (type != CWConstants.SPLIT_MODE_LINE) )
         throw new IllegalArgumentException("Error: You must specify '"+CWConstants.SPLIT_MODE_BINARY+"' or '"+CWConstants.SPLIT_MODE_LINE+"' for typer attribute for the Split action.");
   }

   /**
    * Output the info
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("     Source File          = " + srcFilename);
      UFM.logger.info("     Target Directory     = " + toDir);
      UFM.logger.info("     Size                 = " + size);
      UFM.logger.info("     Type                 = " + type);
      UFM.logger.info("     Prefix               = " + (prefix==null?"":prefix));
      UFM.logger.info("     Extension            = " + (extension==null?"":extension));
      UFM.logger.info("     Create Directory     = " + createDir);
   }
   
   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();
      String file;

      if (srcFilename.indexOf(File.separator) != -1 )
      {
         int x = srcFilename.lastIndexOf(File.separator);

         String dir   = srcFilename.substring(0, x);
         String fname = srcFilename.substring(x+1);

         file = "<"+CWConstants.FILE_ELEMENT+" "+CWConstants.DIR_ATTRIBUTE+"=\""+dir+"\">"+fname+"</"+CWConstants.FILE_ELEMENT+">";
      }
      else
         file = "<"+CWConstants.FILE_ELEMENT+">"+srcFilename+"</"+CWConstants.FILE_ELEMENT+">";
      
      al.add("<"+CWConstants.SPLIT_ELEMENT+" "+CWConstants.TYPE_ATTRIBUTE+"=\""+type+"\" "+CWConstants.SIZE_ATTRIBUTE+"=\""+size+"\" "+CWConstants.TO_DIR_ATTRIBUTE+"=\""+toDir+"\" "+"\" "+CWConstants.PREFIX_ATTRIBUTE+"=\""+prefix+"\" "+"\" "+CWConstants.EXTENSION_ATTRIBUTE+"=\""+extension+"\">"+LINE_SEPARATOR);
      al.add("  "+file+LINE_SEPARATOR);
      al.add("</"+CWConstants.SPLIT_ELEMENT+">"+LINE_SEPARATOR);
      
      return al;
   }

   /**
    * Retrieve the values in this action for the status update.
    * @return s
    */
   public String toStringForStatusUpdate(String event)
   {
      String s;

      s = CWConstants.SPLIT_ELEMENT+", "+labelName+", "+CWConstants.SPLIT_ELEMENT+", "+CWConstants.CREATE_DIR_ATTRIBUTE+"="+createDir+", "+CWConstants.TYPE_ATTRIBUTE+"="+type+", "+CWConstants.SIZE_ATTRIBUTE+"="+size+", srcFilename="+srcFilename+", "+CWConstants.TO_DIR_ATTRIBUTE+"="+toDir+", "+CWConstants.PREFIX_ATTRIBUTE+"="+prefix+", "+CWConstants.EXTENSION_ATTRIBUTE+"="+extension+LINE_SEPARATOR;

      return s;
   }
}
