package com.capitalware.ufm.actions;

import java.io.IOException;

import org.jdom.JDOMException;

import com.capitalware.ufm.action.objects.ExecuteObject;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.handlers.RunJob;
import com.capitalware.ufm.UFM;

/**
 * This class will handle executing a command action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ExecuteAction extends UFMAction
{
   private ExecuteObject   eo;
   private InternalMessageQueue    iq = null;

   /**
    * The constructor
    * @param eo
    */
   public ExecuteAction(ExecuteObject eo, InternalMessageQueue iq)
   {
      super(eo.actionName);
      this.eo = eo;
      this.iq = iq;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      eo.printValues();

      RunJob rj = new RunJob(iq);
      
      try
      {
         rj.launch(eo.xmlFile, eo.paClass.actionProperties, eo.actionName, eo.labelName);
      }
      catch (JDOMException e)
      {
         UFM.logger.fatal("JDOMException: "+e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      catch (IOException e)
      {
         UFM.logger.fatal("IOException: "+e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      finally
      {
         if (!eo.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }

      return exitCode;
   }
}
