package com.capitalware.ufm.actions;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Hashtable;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.handlers.ProcessActions;
import com.capitalware.ufm.action.objects.UFMObject;
import com.capitalware.ufm.action.objects.IfObject;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.objects.FieldOfAction;

/**
 * This class will handle the If action.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class IfAction extends UFMAction
{
   private IfObject           io = null;
   private InternalMessageQueue       iq = null;

   /**
    * The constructor
    * @param io
    * @param iq
    */
   public IfAction(IfObject io, InternalMessageQueue iq)
   {
      super(io.actionName);
      this.io = io;
      this.iq = iq;
   }

   /**
    * Perform the work.
    * @return exitCode
    */
   public int doWork()
   {
      int      exitCode = 0;
      boolean  conditionalTest = false;
      io.printValues();

      UFMObject fmo = UFM.actionObjectValues.get(io.ifLabelName+"_"+io.ifActionName);
      
      if (fmo != null)
      {
         FieldOfAction foa = getFieldFromClass(fmo, io.ifFieldName);
         
         if (foa != null)
         {
            if (foa.value instanceof Integer)
               conditionalTest = conditionalIntTest(foa);
            else if (foa.value instanceof String)
               conditionalTest = conditionalStringTest(foa);
            else if (foa.value instanceof Boolean)
               conditionalTest = conditionalBooleanTest(foa);
            else if (foa.value instanceof Character)
               conditionalTest = conditionalCharacterTest(foa);
            else
            {
               UFM.logger.fatal("If Action: Unknown class: "+foa.value);
               exitCode = 16; 
            }
         
            if (conditionalTest)
            {
               if (io.actionAL != null)
               {
                  /* Clone the array of objects */
                  ArrayList<UFMObject> newActionAL = new ArrayList<UFMObject>();
                  for (int k=0; k < io.actionAL.size(); k++)
                     newActionAL.add((UFMObject)io.actionAL.get(k).clone());

                  ProcessActions pa = new ProcessActions(iq);
                  exitCode = pa.doWork(newActionAL, (Hashtable<String,String>) io.paClass.actionProperties.clone());
               }
            }
            else
            {
               /* the IF failed therefore do the ELSE */
               
               if (io.eo.actionAL != null)
               {
                  /* Clone the array of objects */
                  ArrayList<UFMObject> newActionAL = new ArrayList<UFMObject>();
                  for (int k=0; k < io.actionAL.size(); k++)
                     newActionAL.add((UFMObject)io.actionAL.get(k).clone());

                  ProcessActions pa = new ProcessActions(iq);
                  exitCode = pa.doWork(newActionAL, (Hashtable<String,String>) io.paClass.actionProperties.clone());
               }
               
               if (io.eo.exitRC != 0)
                  exitCode = io.eo.exitRC;

               if (!io.eo.onErrorFail)
                  exitCode = 0; /* reset back to zero */
            }
         }
         else
            exitCode = 16; /* cannot find Action field */
      }
      else
         exitCode = 16; /* cannot find Action in hashtable */

      if (!io.onErrorFail)
         exitCode = 0; /* reset back to zero */
      
      return exitCode;
   }

   /**
    * 
    * @param fmo
    * @param fieldName
    * @return
    */
   private FieldOfAction getFieldFromClass(UFMObject fmo, String fieldName)
   {
      FieldOfAction foa = null;
      Class c = fmo.getClass();
      
      try
      {
         Field f = c.getField(fieldName);
         
         foa = new FieldOfAction();
         foa.name = f.getName();
         foa.type = f.getType().getName();
         foa.value = f.get(fmo);
      }
      catch (SecurityException e)
      {
         UFM.logger.fatal("SecurityException: " + e.getLocalizedMessage());
      }
      catch (NoSuchFieldException e)
      {
         UFM.logger.fatal("NoSuchFieldException: " + e.getLocalizedMessage());
      }
      catch (IllegalArgumentException e)
      {
         UFM.logger.fatal("IllegalArgumentException: " + e.getLocalizedMessage());
      }
      catch (IllegalAccessException e)
      {
         UFM.logger.fatal("IllegalAccessException: " + e.getLocalizedMessage());
      }
      
      return foa;
   }
   
   /**
    * 
    * @param foa
    * @return
    */
   private boolean conditionalIntTest(FieldOfAction foa)
   {
      boolean  conditionalTest = false;
      
      int v = StringUtilities.stringToInt(io.value, 0);
      int fv = foa.getValueInt();

      UFM.logger.info("conditionalIntTest: "+foa.name+" of "+foa.getValueInt()+" "+io.operator+" "+v+" ?");
      
      if ("EQ".equals(io.operator))
      {
         if (fv == v)
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      if ("NE".equals(io.operator))
      {
         if (fv != v)
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      else if ("LT".equals(io.operator))
      {
         if (fv < v)
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      else if ("LE".equals(io.operator))
      {
         if (fv <= v)
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      else if ("GT".equals(io.operator))
      {
         if (fv > v)
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      else if ("GE".equals(io.operator))
      {
         if (fv >= v)
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      
      return conditionalTest;
   }
   
   /**
    * 
    * @param foa
    * @return
    */
   private boolean conditionalStringTest(FieldOfAction foa)
   {
      boolean  conditionalTest = false;

      UFM.logger.info("conditionalStringTest: "+foa.name+" of "+foa.getValueString()+" "+io.operator+" "+io.value+" ?");
      
      if ("EQ".equals(io.operator))
      {
         if (io.value.equalsIgnoreCase(foa.getValueString()))
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      if ("NE".equals(io.operator))
      {
         if (!(io.value.equalsIgnoreCase(foa.getValueString())))
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      
      return conditionalTest;
   }
   
   /**
    * 
    * @param foa
    * @return
    */
   private boolean conditionalBooleanTest(FieldOfAction foa)
   {
      boolean  conditionalTest = false;
      
      boolean v = false;
      if (io.value.equalsIgnoreCase("true"))
         v = true;

      UFM.logger.info("conditionalIntTest: "+foa.name+" of "+foa.getValueBoolean()+" "+io.operator+" "+v+" ?");
      
      if ("EQ".equals(io.operator))
      {
         if ( v == foa.getValueBoolean())
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      if ("NE".equals(io.operator))
      {
         if ( v != foa.getValueBoolean())
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      
      return conditionalTest;
   }
   
   /**
    * 
    * @param foa
    * @return
    */
   private boolean conditionalCharacterTest(FieldOfAction foa)
   {
      boolean  conditionalTest = false;
      
      char v = io.value.charAt(0);

      UFM.logger.info("conditionalIntTest: "+foa.name+" of "+foa.getValueBoolean()+" "+io.operator+" "+v+" ?");
      
      if ("EQ".equals(io.operator))
      {
         if ( v == foa.getValueCharacter())
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      if ("NE".equals(io.operator))
      {
         if ( v != foa.getValueCharacter())
            conditionalTest = true;
         else
            conditionalTest = false;
      }
      
      return conditionalTest;
   }
}
