package com.capitalware.ufm.actions;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.action.objects.MergeObject;
import com.capitalware.ufm.handlers.IO;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.UFMException;

/**
 * This class will handle the Merge action.
 * 
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MergeAction extends UFMAction
{
   private MergeObject mo;

   /**
    * The constructor
    * @param mo
    */
   public MergeAction(MergeObject mo)
   {
      super(mo.actionName);
      this.mo = mo;
   }

   /**
    * Perform the merge work.
    * @return exitCode
    */
   public int doWork()
   {
      int exitCode = 0;
      String toPathAndFilename;
      mo.printValues();

      toPathAndFilename = IO.apendFileToPath(mo.toDir, mo.toFile);
      toPathAndFilename = Tokens.replace(mo.actionName, toPathAndFilename, IO.getFileName(mo.srcFilenames[0]), 1, mo.paClass.actionProperties);

      try
      {
         mo.srcFilenames = IO.buildStringArrayOfFiles(mo.srcFilenames);
         
         IO.mergeFiles(mo.srcFilenames, toPathAndFilename);
         mo.fileCount++;

         UFM.logger.info("Successfully merged files into " + toPathAndFilename);
      }
      catch (UFMException e)
      {
         UFM.logger.fatal("UFMException: "+e.getLocalizedMessage());
         if (e.getCause() != null)
            UFM.logger.fatal(e.getCause().toString());;
         exitCode = 8;
      }
      finally
      {
         if (!mo.onErrorFail)
            exitCode = 0; /* reset back to zero */
      }

      return exitCode;
   }
}
