package com.capitalware.ufm.ftp.objects;

import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;

public class PutObject extends FtpCmdObject
{
   public String  localFilename = null;
   public String  localDir = null;
   public String  remoteFilename = null;
   public String  remoteDir = null;
   //
   public int     fileCount = 0;

   /**
    * The constructor
    */
   public PutObject()
   {
      super();
   }
   
   /**
    * Process tokens in ftp Object's variables
    */
   public void performTokenization(Hashtable<String,String>  properties)
   {
      if (localFilename != null)
         localFilename   = Tokens.replace(localFilename, properties);
      if (localDir != null)
         localDir = Tokens.replace(localDir, properties);
      if (remoteFilename != null)
         remoteFilename   = Tokens.replace(remoteFilename, properties);
      if (remoteDir != null)
         remoteDir = Tokens.replace(remoteDir, properties);
   }

   /**
    * Verify the values in the object.
    */
   public void checkObject()
   {
      if ( (localFilename == null) && (remoteFilename == null) )
         throw new IllegalArgumentException("Error: You must specify a filename for the ftp put command.");
      
      if ( (localFilename != null) && 
           (localFilename.indexOf(File.separator) != -1) )
      {
         int x = localFilename.lastIndexOf(File.separator);
         
         localDir   = localFilename.substring(0, x);
         localFilename = localFilename.substring(x+1);
      }

      if (localFilename == null)
         localFilename = remoteFilename;
      
      if (localDir == null)
         throw new IllegalArgumentException("Error: You must specify a local directory for the ftp put command.");
   }

   /**
    * Output the info
    * @param logger
    */
   public void printValues()
   {
      UFM.logger.info("         Local FileName   = " + (localFilename == null?"":localFilename));
      UFM.logger.info("         Local Directory  = " + (localDir == null?"":localDir));
      UFM.logger.info("         Remote FileName  = " + (remoteFilename == null?"":remoteFilename));
      UFM.logger.info("         Remote Directory = " + (remoteDir == null?"":remoteDir));
   }

   /**
    * Convert the object to a string.
    * @return s
    */
   public ArrayList<String> toStringArray()
   {
      ArrayList<String> al = new ArrayList<String>();
      String file;
      
      if (localFilename.indexOf(File.separator) != -1 )
      {
         int x = localFilename.lastIndexOf(File.separator);

         String dir   = localFilename.substring(0, x);
         String fname = localFilename.substring(x+1);

         file = "<"+CWConstants.FILE_ELEMENT+" "+CWConstants.DIR_ATTRIBUTE+"=\""+dir+"\">"+fname+"</"+CWConstants.FILE_ELEMENT+">";
      }
      else
         file = "<"+CWConstants.FILE_ELEMENT+">"+localFilename+"</"+CWConstants.FILE_ELEMENT+">";

      if (remoteFilename == null)
         al.add("<"+CWConstants.FTP_PUT_ELEMENT+" "+CWConstants.REMOTE_DIR_ATTRIBUTE+"="+remoteDir+"\">"+LINE_SEPARATOR);
      else
         al.add("<"+CWConstants.FTP_PUT_ELEMENT+" "+CWConstants.REMOTE_DIR_ATTRIBUTE+"="+remoteDir+"\" "+CWConstants.TO_FILE_ATTRIBUTE+"="+remoteFilename+"\">"+LINE_SEPARATOR);

      al.add("  "+file+LINE_SEPARATOR);
      al.add("</"+CWConstants.FTP_PUT_ELEMENT+">"+LINE_SEPARATOR);

      return al;
   }
}
