package com.capitalware.ufm.other;

import java.net.InetAddress;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.StringUtilities;
import com.capitalware.ufm.mq.MQLevel;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class logs general / common information.
 *
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.2
 * @license Apache 2 License
 */
public class GeneralInfo
{
   /**
    * The constructor - make sure nothing instantiates this class.
    * It must be accessed in a static way.
    */
   private GeneralInfo()
   {}

   /**
    * Dump the header information.
    * @param name
    * @param logger
    */
   public static void dumpHeader()
   {
      UFM.logger.info("----------------------------------------------------------------------");
      UFM.logger.info(UFM.WHO_AM_I + " version " + CWConstants.VERSION + " is now starting.");

      UFM.logger.info("System Information:");
      try
      {
         UFM.logger.info("     JVM                  = " + System.getProperty("java.vendor") + " v"  + System.getProperty("java.version"));
         UFM.logger.info("     OS                   = " + System.getProperty("os.name"));
         UFM.logger.info("     user.name            = " + System.getProperty("user.name") );
         UFM.logger.info("     user.home            = " + System.getProperty("user.home"));
         UFM.logger.info("     user.dir             = " + System.getProperty("user.dir"));
         UFM.logger.info("     java.io.tmpdir       = " + System.getProperty("java.io.tmpdir"));
         UFM.logger.info("     Hostname             = " + InetAddress.getLocalHost().getHostName());
         UFM.logger.info("     HostAddress          = " + InetAddress.getLocalHost().getHostAddress());
         UFM.logger.info("     CanonicalHostname    = " + InetAddress.getLocalHost().getCanonicalHostName());
         UFM.logger.info("     UseFileLocking       = " + UFM.useFileLocking);
         
         try 
         {
            ClassLoader classLoader = ClassLoader.getSystemClassLoader();
            Class.forName("com.ibm.mq.MQJavaLevel", false, classLoader);
            MQLevel mql = new MQLevel();
            UFM.logger.info("     MQ JAR Version       = " + mql.getName()+" V"+mql.getVersion());
           
            // it exists on the classpath
         } 
         catch(ClassNotFoundException e) 
         {
            UFM.logger.debug("MQ is not in the CLASSPATH.");
         }
      }
      catch (Exception e)
      {
         UFM.logger.fatal("Exception: " + e );
      }
   }

   /**
    * Dump the header information.
    * @param name
    * @param logger
    */
   public static void globalValues(Hashtable<String,String> userProperties)
   {
      UFM.logger.info("User Global Values:");
      
      if (!(userProperties.isEmpty()))
      {
         Enumeration<String> keysEnum = userProperties.keys();
         Vector<String> keyList = new Vector<String>();
         while(keysEnum.hasMoreElements())
         {
            String item = (String)keysEnum.nextElement();
            // Do not add hostname or ipaddress
            if ( !((CWConstants.HOSTNAME_TOKEN.equals(item)) || 
                   (CWConstants.IPADDRESS_TOKEN.equals(item))) )
                keyList.add(item);
         }
         Collections.sort(keyList);
         
         if (keyList.isEmpty())
         {
            UFM.logger.info("     <None>");
         }
         else
         {
            Enumeration<String> names = keyList.elements();
            
            while (names.hasMoreElements())
            {
               String name = names.nextElement();
               UFM.logger.info("     "+StringUtilities.padRight(name, " ", 21)+ "= " + userProperties.get(name));
            }
         }
      }
   }
}
