package com.capitalware.ufm.schedule;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.InternalMessageQueue;
import com.capitalware.ufm.internal.messages.UpdateStatus;
import com.capitalware.ufm.objects.CWConstants;

/**
 * This class will handle a particular event launching the associated Workflow.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ProcessScheduleEntry implements ScheduleListener
{
   private String          actionName = ""; 
   private String          labelName = "";
   private Holiday[]       holidays = null;
   private InternalMessageQueue    iq = null;
   
   /**
    * The constructor
    * @param actionName
    * @param labelName
    * @param holidays
    * @param iq
    */
   public ProcessScheduleEntry(String actionName, String labelName, Holiday[] holidays, InternalMessageQueue iq)
   {
      super();
      this.actionName = actionName;
      this.labelName = labelName;
      this.holidays = holidays;
      this.iq = iq;
   }
   
   /**
    * Process the event
    * @param entry
    */
   public void handleEvent(ScheduleEntry entry)
   {
      SimpleDateFormat dateTimeFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
      boolean doEvent = true;
      Holiday holiday = null;
      Calendar cal = Calendar.getInstance();
      int day = cal.get(Calendar.DAY_OF_MONTH);
      int month = cal.get(Calendar.MONTH);
      int year = cal.get(Calendar.YEAR);
      String msg = "Schedule Action is starting: "+entry.getName()+" [" + entry.getFile()+"]";
      
      if (holidays != null)
      {
         for (int j=0; j < holidays.length; j++)
         {
            holiday = holidays[j];
            
            if ( ( (holiday.year == -1) || (holiday.year == year) )    &&
                 ( (holiday.month == -1) || (holiday.month == month) ) &&
                 ( (holiday.day == -1) || (holiday.day == day) ) )
            {
               doEvent = false;
               msg = "Holiday: skipping: "+entry.getName()+" [" + entry.getFile()+"]";
            }
         }
      }
      
      if (iq != null)
         iq.addMessage(new UpdateStatus((doEvent?0:4)+"", dateTimeFormat.format(new Date())+", "+(doEvent?0:4)+", "+actionName+", "+labelName+", Schedule, Event="+msg+",\n"));

      UFM.logger.info(msg);
      
      if (doEvent)
      {
         /* To expedite the work - launch Workflow in its own thread. */ 
         Thread t = new Thread(new Runner(entry.getFile()));
         t.start();
      }
   }
   
   /**
    * This class will build and run the Workflow in a separate thread.
    */
   class Runner implements Runnable
   {
      private String xmlFileName;

      /**
       * The constructor
       * @param xmlFileName
       */
      public Runner(String xmlFileName)
      {
         super();
         this.xmlFileName = xmlFileName;
      }
      
      /**
       * Start of thread.
       */
      public void run()
      {
         UFM.logger.info("Runner: Starting a new thread to run the Workflow: "+xmlFileName);
         UFM fm = new UFM();
         fm.setXmlFilename(xmlFileName);
         fm.doWork(CWConstants.ONE_PASS, false);
      }
   }
}
