package com.capitalware.ufm.xml;

import org.jdom.*;
import org.jdom.input.*;

import com.capitalware.ufm.UFM;
import com.capitalware.ufm.handlers.Tokens;
import com.capitalware.ufm.objects.CWConstants;
import com.capitalware.ufm.objects.JobObject;
import com.capitalware.ufm.xml.DTDs.ResolveLocalDTD;

import java.util.*;
import java.io.*;

/**
 * This class handles the reading and parsing of Job XML file and parse parameters
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class JobXml extends UFMXmlAbstract
{
   private JobObject                   _job;
   private Hashtable<String,String>    properties = null;
   private String                      actionName = ""; 
   private String                      actionLabel = "";

   /**
    * The constructor
    * @param logger
    */
   public JobXml()
   {
      super();
   }

   /**
    * Read Job XML file and set parameters
    *
    * @param xmlFilename
    *           Filename of job.xml file
    * @throws IOException
    */
   public void readXMLFile(String jobXmlFile) throws JDOMException, IOException
   {
      SAXBuilder builder = new SAXBuilder(true);
      // handle the ability to pull DTD from Jar if needed
      builder.setEntityResolver(new ResolveLocalDTD(CWConstants.JOBS_DTD));
      Document doc = builder.build(CWConstants.JOBS_PATH + jobXmlFile);
      Element rootElement = doc.getRootElement();
      // get main element
      Element jobElement = rootElement.getChild(CWConstants.JOB_ELEMENT);
      String jobNameAttr = null;
      if (jobElement.getAttribute(CWConstants.NAME_ATTRIBUTE) != null)
         jobNameAttr = Tokens.replace("Execute", jobElement.getAttribute(CWConstants.NAME_ATTRIBUTE).getValue(), properties);

      Element cmdElement = jobElement.getChild(CWConstants.COMMAND_ELEMENT);
      String cmd = Tokens.replace("Execute", cmdElement.getTextTrim(), properties);

      String wait= cmdElement.getAttribute(CWConstants.WAIT_ATTRIBUTE).getValue();

      String workingDirectory = null;
      if ( jobElement.getChild(CWConstants.DIRECTORY_ELEMENT) != null)
         workingDirectory = Tokens.replace("Execute", jobElement.getChild(CWConstants.DIRECTORY_ELEMENT).getTextTrim(), properties);

      List parmList = jobElement.getChildren(CWConstants.PARM_ELEMENT);
      ArrayList<String> jobAL = new ArrayList<String>();
      Iterator it = parmList.iterator();
      while (it.hasNext())
      {
         Element parmElement = (Element) it.next();
         jobAL.add(Tokens.replace("Execute", parmElement.getTextTrim(), properties));
      }

      _job = new JobObject(jobNameAttr, cmd, jobAL, workingDirectory, wait, actionName, actionLabel);

      UFM.logger.info("Values used for this Job:");
      UFM.logger.info("    Job Name            = " + (jobNameAttr == null?"":jobNameAttr));
      UFM.logger.info("    Command             = " + cmd);
      for (int i=0; i < jobAL.size(); i++)
         UFM.logger.info("    Parm["+(i+1)+"]             = " + jobAL.get(i));
      UFM.logger.info("    Working Directory   = " + (workingDirectory == null?"":workingDirectory));
      UFM.logger.info("    Wait for job        = " + wait);
   }

   /**
    * Retrieve the job structure
    * @return job
    */
   public JobObject getJob()
   {
      return _job;
   }
   
   /**
    * Set the properties
    * @param properties
    */
   public void setProperties(Hashtable<String,String>  properties)
   {
      this.properties = properties;
   }

   /**
    * Set what action is the parent.
    * @param actionName
    * @param actionLabel
    */
   public void setParentage(String actionName, String actionLabel)
   {
      this.actionName = actionName;
      this.actionLabel = actionLabel;
   }
}
